<?php
/**
 * CryptoPhoto Special User Management page
 * 
 * @file
 * @ingroup Extensions
 */

class CPUsersPager extends TablePager {
    // variable that holds the default user data we show to the administrator
    // an array of $uid_data
    private $uids_data = array();
    // variable used to build default data for each user
    // an array containing:
    //   uid - user id
    //   CryptoPhoto enabled flag - 1 | 0 | d (d means that the user is either required or not allowed to use Cryptophoto - so not a user choice)
    //   CryptoPhoto Admin Override flag - 1 | 0 (disable Cryptophoto authentication for the user's next login)
    private $uid_data = array();
    // variable used to read user preferences and groups
    private $user;
    // filter for user name/email
    private $filter;
    // database access object
    private $dbr;

    function __construct($filter = '') {
        parent::__construct();

        // create a new User object, we'll load user data later
        $this->user = new User();

        // save the filter we received, if any
        $this->filter = $filter;

      	// get database access
  		$this->dbr = wfGetDB(DB_SLAVE);
    }

    /**
    * What we need from the database
    */
    function getQueryInfo() {
        // User table and necessary fields
        $dbs = array(
            'tables' => 'user',
            'fields' => 'user_id, user_name, user_email, user_touched'
        );

        // add a LIKE condition if a filter was specified
        if ($this->filter != '') {
            $like = $this->dbr->buildLike($this->dbr->anyString(), $this->filter, $this->dbr->anyString());
            $dbs['conds'] = 'user_name' . $like . ' OR user_email' . $like;
        }

        return $dbs;
    }

    function getIndexField() {
        return 'user_id';
    }

    function isFieldSortable($field) {
        return 1;
    }

    /**
    * Format table cells
    */
    function formatValue($name, $value) {
        global $wgCryptoPhotoConfig;

        if ($name == 'user_id') {
            // new table row. save previous user default data
            if (count($this->uid_data)) {
                array_push($this->uids_data, implode(',', $this->uid_data));
                $this->uid_data = array();
            }

            // reset and load user data
            $this->user->loadDefaults();
            $this->user->setId($value);
            array_push($this->uid_data, $value);
        } elseif ($name == 'user_touched') {
            // date for last login/last change for an article, etc
            global $wgLang;
            $lastLogin = $wgLang->timeanddate( wfTimestamp( TS_MW, $value ), true );
            $daysAgo = $wgLang->formatNum( round( ( time() - wfTimestamp( TS_UNIX, $value ) ) / 3600 / 24, 2 ), 2 );
            return "$lastLogin ($daysAgo " . wfMessage('cpusers-days-ago')->text() . ")";
        } elseif ($name == 'cp_enabled') {
            // here we build the checkbox to allow the administrator to enable/disable CryptoPhoto
            // for an user. The checkbox is disabled if the user is part of a mandatory group or it is not part
            // of an allowed group (changing the checkbox state will have no meaning because the user is either
            // required or not allowed to use CryptoPhoto - so not much choice in this case)
            
            // checkbox state (on/off)
            $cpAuth = 0;
            // checkbox disabled state
            $cpDisabled = 0;
            // optional message if the checkbox is disabled (currently 'mandatory' or 'not allowed')
            $cpMsg = '';

            // check if the user is part of the mandatory groups first
            if (count($wgCryptoPhotoConfig['mandatory_groups']) ?
                count(array_intersect($wgCryptoPhotoConfig['mandatory_groups'], $this->user->getEffectiveGroups())) : 0) {
              // force CryptoPhoto authentication
              $cpAuth = 1;
          	  $cpDisabled = 1;
          	  $cpMsg = wfMessage('cpusers-cpe-mandatory')->text();
            } else {
              // check if the user is part of the allowed groups
              if (count($wgCryptoPhotoConfig['groups']) ?
                  count(array_intersect($wgCryptoPhotoConfig['groups'], $this->user->getEffectiveGroups())) : 0) {
                // get user option for using CryptoPhoto
                $cpAuth = $this->user->getOption('cryptophoto_enabled', 0);
              } else {
              	$cpDisabled = 1;
              	$cpMsg = wfMessage('cpusers-cpe-not-allowed')->text();;
              }
            }

            if ($cpDisabled) {
                // if the checkbox is disabled we're not interested in the default state (there is no reason to change it)
                // also, disabled HTML fields do not get POSTed back from the browser
                array_push($this->uid_data, 'd');
            } else {
                // save the user option to use CryptoPhoto otherwise
                array_push($this->uid_data, $cpAuth);
            }

            // build and return the 'CryptoPhoto enabled' checkbox
            return '<input type="checkbox" name="cp_enabled[]" value="' . $this->user->getId() . '"' .
                   ($cpAuth ? ' checked="checked"' : '') .
                   ($cpDisabled ? ' readonly disabled' : '') .
                   ($cpMsg != '' ? '>(' . $cpMsg . ')</input>' : '/>');

        } elseif ($name == 'cp_admin_override') {
            // read the CryptoPhoto Admin Override options (this is set by the administrator)
            $cpao = $this->user->getOption('cryptophoto_admin_override', 0);

            // store the default value we just read
            array_push($this->uid_data, $cpao);

            // build and return the 'CryptoPhoto Admin Override' checkbox
            return '<input type="checkbox" name="cp_admin_override[]" value="' . $this->user->getId() . '"' .
                   ($cpao ? ' checked="checked"' : '') .
                   '/>';
        } elseif ($name == 'user_groups') {
            // return the user effective groups
            return implode(', ', array_diff($this->user->getEffectiveGroups(), array('*')));
        }

        return "$value";
    }

    function getDefaultSort() {
        return 'user_id';
    }

    function getFieldNames() {
        return array(
            'user_id' => wfMessage('cpusers-field-userid')->text(),
            'user_name' => wfMessage('cpusers-field-username')->text(),
            'cp_enabled' => wfMessage('cpusers-field-cpenabled')->text(),
            'cp_admin_override' => wfMessage('cpusers-field-cpao')->text(),
            'user_groups' => wfMessage('cpusers-field-usergroups')->text(),
            'user_email' => wfMessage('cpusers-field-useremail')->text(),
            'user_touched' => wfMessage('cpusers-field-usertouched')->text()
        );
    }

    /**
    * Get the default data we show to the administrator
    */
    function getUidsData() {
        array_push($this->uids_data, implode(',', $this->uid_data));
        return implode(';', $this->uids_data);
    }
}

class SpecialCryptoPhotoUsers extends SpecialPage {
    function __construct() {
        parent::__construct( 'CryptoPhotoUsers', '', false );
    }

    function execute( $par ) {
        global $wgUser, $mediaWiki, $wgRequest, $wgOut, $wgCryptoPhotoConfig, $IP, $wgServer, $wgScriptPath, $wgSecretKey;

        // test for administrator login
        if ($wgUser->isLoggedIn() && in_array('sysop', $wgUser->getEffectiveGroups())) {
            // load CryptoPhoto settings
            CryptoPhotoDatabaseSupport::loadSettings();

            // prepare output page
            $this->setHeaders();
            $this->outputHeader();

            // get WebRequest object
            $req = $this->getRequest();

            // get OutputPage object
            $out = $this->getOutput();

            // get the user input filter
            $filter = $req->getVal('filter', '');
            
            // check if something was posted back yo us
            if (isset($_POST['save'])) {
                // administrator wants to save choices

                // get the default user data we sent and split it
                $uids_data = explode(';', $req->getVal('uids_data', ''));
                
                // get the administrator choices for which users should have CryptoPhoto enabled
                $cp_enabled = $req->getArray('cp_enabled', array());
                // get the administrator choices for which users should bypass CryptoPhoto authentication on next login
                $cp_admin_override = $req->getArray('cp_admin_override', array());

                // User object we'll use to save user settings if required
                $user = null;

                // process the default user data we sent
                foreach ($uids_data as $uid_data) {
                    // get user ID, CryptoPhoto enabled flag, CryptoPhoto Admin Override flag we sent
                    list($uid, $cpe, $cpao) = explode(',', $uid_data);

                    // flags to be set if something has changed
                    $changed_cp = 0; $changed_cpao = 0;
                    
                    // test if the 'CryptoPhoto enabled' checkbox was disabled. If it was we're not interested in the administrator input
                    // see a detailed explanation in CPUserPage::formatValue, 'cp_enabled' field
                    if ($cpe != 'd') {
                        // checkbox was enabled, see if the value has changed
                        $_cpe = in_array($uid, $cp_enabled) ? 1 : 0;
                        if ($cpe != $_cpe) $changed_cp = 1;
                    }

                    // see if the CryptoPhoto Admin Override checkbox was changed
                    $_cpao = in_array($uid, $cp_admin_override) ? 1 : 0;
                    if ($cpao != $_cpao) $changed_cpao = 1;

                    // save user settings if needed
                    if ($changed_cp || $changed_cpao) {
                        if (!$user) $user = new User();
                        
                        $user->setId($uid);
                        $user->loadFromId();

                        if ($changed_cp) $user->setOption('cryptophoto_enabled', $_cpe);
                        if ($changed_cpao) $user->setOption('cryptophoto_admin_override', $_cpao);

                        $user->saveSettings();
                    }
                }

                // inform the administrator that the changes were saved
                $out->addHTML('<div class="successbox">' . wfMessage('cpusers-saved-success')->text() . '</div>');				
            } elseif (isset($_POST['clear_filter'])) {
                // administrator wants to clear the filter
                $filter = '';
            }

            // build the CryptoPhoto Users Management table
            $up = new CPUsersPager($filter);

         	$wgOut->addHTML(
         		Html::rawElement(
                    // add form for submitting changes and applying/clearing the filter
                    'form',
                    array(
                        'method' => 'post',
                        'action' => $this->getRequest()->getRequestUrl(),
                    ),
                    "\n" .
                    '<p>' .
                        '<input type="submit" name="save" value="' . wfMessage('cpusers-save-button')->text() .'"/>' .
                        '&nbsp; | &nbsp' .
                        wfMessage('cpusers-filter-label')->text() . '<input type="text" name="filter" id="filter" value="' . $filter . '" />' .
                        '<input type="submit" name="apply_filter" value="' . wfMessage('cpusers-filter-apply-button')->text() .'"/>' .
                        '&nbsp' .
                        '<input type="submit" name="clear_filter" value="' . wfMessage('cpusers-filter-clear-button')->text() .'"/>' .
                    '</p>' .
                    // add users table
                    $up->getBody() .
                    // add the default user data
                    '<input type="hidden" name="uids_data" value="' . $up->getUidsData() . '"/>' .
                    // add table page navigation (only shown if needed)
                    $up->getNavigationBar()					 
                ) . "\n" .
                '<p></p>' .
                // add number of records/page selection
                $up->getLimitForm()
            );
        } else {
            // attempt to reach the CryptoPhoto Users Management page, but without administrator rights
            $wgOut->addWikiText(wfMessage('cp-error-not-admin')->text());
        }
    }  
}
